#include <creaImageIOWxGimmickReaderDialog.h>
#include <creaImageIOSystem.h>
#include <creaImageIOGimmick.h>
#ifdef _DEBUG
#define new DEBUG_NEW
#endif
namespace creaImageIO
{
  // CTor
  WxGimmickReaderDialog::WxGimmickReaderDialog(wxWindow *parent, 
					       wxWindowID id,
					       const std::string i_namedescp, 
					       const std::string i_namedb,
					       wxString title,
					       const wxPoint& pos,
					       const wxSize& size,
					       int min_dim,
					       int max_dim,
					       int output_dim,  // never used ?!? // JPR
					       int threads)
 :   wxDialog( parent, 
		  id, 
		  title,
		  pos,
		  size,
		  wxRESIZE_BORDER | 
		  wxSYSTEM_MENU   |
		  wxCLOSE_BOX     |
		  wxMAXIMIZE_BOX  | 
		  wxMINIMIZE_BOX  | 
		  wxCAPTION  
	       ),
 //   mGimmick(0),
     mView(0)
  {
    GimmickDebugMessage(1,"WxGimmickReaderDialog::WxGimmickReaderDialog"
			<<std::endl);
    mtopsizer = new wxBoxSizer(wxVERTICAL);

    try {

	  mGimmick = boost::shared_ptr<Gimmick>(new Gimmick());
      mGimmick->Initialize(i_namedescp,i_namedb);

      mView = new WxGimmickView(mGimmick,
				this,
				TVID,
				wxDefaultPosition,
				size,
				min_dim,
				max_dim,
				threads);
      mView->Initialize();
	   // Connect the AddProgress callback
      mView->ConnectValidationObserver
      		( boost::bind( &WxGimmickReaderDialog::OnValid , this, _1 ) );
    }
    catch (crea::Exception e)
    {
      e.Print();
      return;
    }

    mtopsizer->Add( mView,1,wxGROW,0);

    wxSizer* bsizer = this->CreateSeparatedButtonSizer(wxOK|wxCANCEL);
	/*mOkButton = new wxButton(this, wxID_OK, _T("OK"), wxPoint(170,50));
	mCancelButton = new wxButton(this, wxID_CANCEL, _T("CANCEL"), wxPoint(210,50));
	*/	mOkButton = (wxButton*)FindWindowById(GetAffirmativeId(), this);
    mCancelButton = (wxButton*)FindWindowById(GetEscapeId(),      this);

    mOkButton->Enable(false);
    mtopsizer->Add ( bsizer, 0, wxGROW );

    SetSizer( mtopsizer );     
 
    Layout(); 
  }

  /// Destructor
  WxGimmickReaderDialog::~WxGimmickReaderDialog()
  {
    GimmickDebugMessage(1,"WxGimmickReaderDialog::~WxGimmickReaderDialog"
			<<std::endl);
    if (mView) 
      {
        delete mView;
      }
    if (mGimmick) 
      {
		mGimmick->Finalize();
      }

  }

  ///Callback method on a selection
  void WxGimmickReaderDialog::OnValid(bool t)
  {
    mOkButton->Enable(t);
  }

  //================================================================
  //BEGIN_EVENT_TABLE(WxGimmickReaderDialog, wxDialog)
  //END_EVENT_TABLE()
  //================================================================

} // EO namespace creaImageIO

